@@ -1,4 +1,4 @@
-/* $Id: directory.c 1590 2004-03-15 12:17:20Z twogood $ */
+/* $Id$ */
 #include "internal.h"
 #include "log.h"
 
@@ -23,7 +23,7 @@ const char* unshield_directory_name(Unshield* unshield, int index)
     Header* header = unshield->header_list;
 
     if (index < (int)header->cab.directory_count)
-      return (const char*)(
+      return unshield_get_utf8_string(header, 
           header->data +
           header->common.cab_descriptor_offset +
           header->cab.file_table_offset +
