@@ -15,6 +15,9 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/param.h>    /* for MIN(a,b) */
+#ifndef MIN /* missing in some platforms */
+#define MIN(a,b) (((a)<(b))?(a):(b))
+#endif
 #include <zlib.h>
 
 #define VERBOSE 3
@@ -267,11 +270,14 @@ static int unshield_uncompress_old(Byte *dest, uLong *destLen, Byte *source, uLo
     if (err != Z_OK) 
       return err;
 
-    err = inflate(&stream, Z_BLOCK);
-    if (err != Z_OK) 
+    while (stream.avail_in > 1)
     {
-      inflateEnd(&stream);
-      return err;
+      err = inflate(&stream, Z_BLOCK);
+      if (err != Z_OK)
+      {
+        inflateEnd(&stream);
+        return err;
+      }
     }
 
     *destLen = stream.total_out;
@@ -524,6 +530,11 @@ static bool unshield_reader_read(UnshieldReader* reader, void* buffer, size_t si
     unshield_trace("Trying to read 0x%x bytes from offset %08x in volume %i", 
         bytes_to_read, ftell(reader->volume_file), reader->volume);
 #endif
+    if (bytes_to_read == 0)
+    {
+      unshield_error("bytes_to_read can't be zero");
+      goto exit;
+    }
 
     if (bytes_to_read != fread(p, 1, bytes_to_read, reader->volume_file))
     {
@@ -593,7 +604,7 @@ static UnshieldReader* unshield_reader_create(/*{{{*/
     }
 
     /* Start with the correct volume for IS5 cabinets */
-    if (reader->unshield->header_list->major_version == 5 &&
+    if (reader->unshield->header_list->major_version <= 5 &&
         index > (int)reader->volume_header.last_file_index)
     {
       unshield_trace("Trying next volume...");
@@ -634,7 +645,7 @@ bool unshield_file_save (Unshield* unshield, int index, const char* filename)/*{
   FILE* output = NULL;
   unsigned char* input_buffer   = (unsigned char*)malloc(BUFFER_SIZE+1);
   unsigned char* output_buffer  = (unsigned char*)malloc(BUFFER_SIZE);
-  int bytes_left;
+  unsigned int bytes_left;
   uLong total_written = 0;
   UnshieldReader* reader = NULL;
   FileDescriptor* file_descriptor;
@@ -678,7 +689,7 @@ bool unshield_file_save (Unshield* unshield, int index, const char* filename)/*{
 
   if (filename) 
   {
-    output = fopen(filename, "w");
+    output = fopen(filename, "wb");
     if (!output)
     {
       unshield_error("Failed to open output file '%s'", filename);
@@ -705,14 +716,19 @@ bool unshield_file_save (Unshield* unshield, int index, const char* filename)/*{
 
       if (!unshield_reader_read(reader, &bytes_to_read, sizeof(bytes_to_read)))
       {
-#if VERBOSE
-        unshield_error("Failed to read %i bytes of file %i (%s) from input cabinet file %i", 
+        unshield_error("Failed to read %i bytes of file %i (%s) from input cabinet file %i",
             sizeof(bytes_to_read), index, unshield_file_name(unshield, index), file_descriptor->volume);
-#endif
         goto exit;
       }
 
       bytes_to_read = letoh16(bytes_to_read);
+      if (bytes_to_read == 0)
+      {
+        unshield_error("bytes_to_read can't be zero");
+        unshield_error("HINT: Try unshield_file_save_old() or -O command line parameter!");
+        goto exit;
+      }
+
       if (!unshield_reader_read(reader, input_buffer, bytes_to_read))
       {
 #if VERBOSE
@@ -731,6 +747,10 @@ bool unshield_file_save (Unshield* unshield, int index, const char* filename)/*{
       {
         unshield_error("Decompression failed with code %i. bytes_to_read=%i, volume_bytes_left=%i, volume=%i, read_bytes=%i", 
             result, bytes_to_read, reader->volume_bytes_left, file_descriptor->volume, read_bytes);
+        if (result == Z_DATA_ERROR)
+        {
+          unshield_error("HINT: Try unshield_file_save_old() or -O command line parameter!");
+        }
         goto exit;
       }
 
@@ -831,7 +851,7 @@ bool unshield_file_save_raw(Unshield* unshield, int index, const char* filename)
   FILE* output = NULL;
   unsigned char* input_buffer   = (unsigned char*)malloc(BUFFER_SIZE);
   unsigned char* output_buffer  = (unsigned char*)malloc(BUFFER_SIZE);
-  int bytes_left;
+  unsigned int bytes_left;
   UnshieldReader* reader = NULL;
   FileDescriptor* file_descriptor;
 
@@ -871,7 +891,7 @@ bool unshield_file_save_raw(Unshield* unshield, int index, const char* filename)
 
   if (filename) 
   {
-    output = fopen(filename, "w");
+    output = fopen(filename, "wb");
     if (!output)
     {
       unshield_error("Failed to open output file '%s'", filename);
@@ -947,7 +967,7 @@ bool unshield_file_save_old(Unshield* unshield, int index, const char* filename)
   size_t input_buffer_size = BUFFER_SIZE;
   unsigned char* input_buffer   = (unsigned char*)malloc(BUFFER_SIZE);
   unsigned char* output_buffer  = (unsigned char*)malloc(BUFFER_SIZE);
-  int bytes_left;
+  unsigned int bytes_left;
   uLong total_written = 0;
   UnshieldReader* reader = NULL;
   FileDescriptor* file_descriptor;
@@ -988,7 +1008,7 @@ bool unshield_file_save_old(Unshield* unshield, int index, const char* filename)
 
   if (filename) 
   {
-    output = fopen(filename, "w");
+    output = fopen(filename, "wb");
     if (!output)
     {
       unshield_error("Failed to open output file '%s'", filename);
@@ -1008,6 +1028,13 @@ bool unshield_file_save_old(Unshield* unshield, int index, const char* filename)
     uLong bytes_to_write = 0;
     int result;
 
+    if (reader->volume_bytes_left == 0 && !unshield_reader_open_volume(reader, reader->volume + 1))
+    {
+      unshield_error("Failed to open volume %i to read %i more bytes",
+                     reader->volume + 1, bytes_left);
+      goto exit;
+    }
+
     if (file_descriptor->flags & FILE_COMPRESSED)
     {
       static const uint8_t END_OF_CHUNK[4] = { 0x00, 0x00, 0xff, 0xff };
@@ -1050,6 +1077,33 @@ bool unshield_file_save_old(Unshield* unshield, int index, const char* filename)
 
         chunk_size = match - chunk_buffer;
 
+        /*
+           Detect when the chunk actually contains the end of chunk marker.
+
+           Needed by Qtime.smk from "The Feeble Files - spanish version".
+
+           The first bit of a compressed block is always zero, so we apply this
+           workaround if it's a one.
+
+           A possibly more proper fix for this would be to have
+           unshield_uncompress_old eat compressed data and discard chunk
+           markers inbetween.
+           */
+        while ((chunk_size + sizeof(END_OF_CHUNK)) < input_size &&
+            chunk_buffer[chunk_size + sizeof(END_OF_CHUNK)] & 1)
+        {
+          unshield_warning("It seems like we have an end of chunk marker inside of a chunk.");
+          chunk_size += sizeof(END_OF_CHUNK);
+          match = find_bytes(chunk_buffer + chunk_size, input_size - chunk_size, END_OF_CHUNK, sizeof(END_OF_CHUNK));
+          if (!match)
+          {
+            unshield_error("Could not find end of chunk for file %i (%s) from input cabinet file %i",
+                index, unshield_file_name(unshield, index), file_descriptor->volume);
+            goto exit;
+          }
+          chunk_size = match - chunk_buffer;
+        }
+
 #if VERBOSE >= 3
         unshield_trace("chunk_size = 0x%x", chunk_size);
 #endif
@@ -1063,7 +1117,7 @@ bool unshield_file_save_old(Unshield* unshield, int index, const char* filename)
 
         if (Z_OK != result)
         {
-          unshield_error("Decompression failed with code %i. input_size=%i, volume_bytes_left=%i, volume=%i, read_bytes=%i", 
+          unshield_error("Decompression failed with code %i. input_size=%i, volume_bytes_left=%i, volume=%i, read_bytes=%i",
               result, input_size, reader->volume_bytes_left, file_descriptor->volume, read_bytes);
           goto exit;
         }
