@@ -1,4 +1,5 @@
 #define _BSD_SOURCE 1
+#define _DEFAULT_SOURCE 1
 #include "internal.h"
 #include "log.h"
 #include "convert_utf/ConvertUTF.h"
@@ -7,6 +8,17 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include <unistd.h>
+
+#ifdef _WIN32
+  #define realpath(N,R) _fullpath((R),(N),_MAX_PATH)
+  #include <direct.h>
+  #ifndef PATH_MAX
+    #define PATH_MAX _MAX_PATH
+  #endif
+#else
+  #include <limits.h>
+#endif
 
 #define VERBOSE 0
 
@@ -15,13 +27,35 @@ FILE* unshield_fopen_for_reading(Unshield* unshield, int index, const char* suff
   if (unshield && unshield->filename_pattern)
   {
     FILE* result = NULL;
-    char filename[256];
-    char dirname[256];
+    char* filename = NULL;
+    char* dirname = NULL;
     char * p = strrchr(unshield->filename_pattern, '/');
     const char *q;
     struct dirent *dent = NULL;
-    DIR *sourcedir;
-    snprintf(filename, sizeof(filename), unshield->filename_pattern, index, suffix);
+    DIR *sourcedir = NULL;
+    long int path_max;
+
+    #ifdef PATH_MAX
+    path_max = PATH_MAX;
+    #else
+    path_max = pathconf(unshield->filename_pattern, _PC_PATH_MAX);
+    if (path_max <= 0)
+      path_max = 4096;
+    #endif
+
+    dirname = malloc(path_max);
+    filename = malloc(path_max);
+    if (filename == NULL || dirname == NULL)
+    {
+      unshield_error("Unable to allocate memory.\n");
+      goto exit;
+    }
+
+    if(snprintf(filename, path_max, unshield->filename_pattern, index, suffix)>=path_max)
+    {
+      unshield_error("Pathname exceeds system limits.\n");
+      goto exit;
+    }
     q=strrchr(filename,'/');
     if (q)
       q++;
@@ -30,11 +64,11 @@ FILE* unshield_fopen_for_reading(Unshield* unshield, int index, const char* suff
 
     if (p)
     {
-      strncpy( dirname, unshield->filename_pattern,sizeof(dirname));
-      if ((unsigned int)(p-unshield->filename_pattern) > sizeof(dirname))
+      strncpy( dirname, unshield->filename_pattern,path_max);
+      if ((unsigned int)(p-unshield->filename_pattern) > path_max)
       {
         unshield_trace("WARN: size\n");
-        dirname[sizeof(dirname)-1]=0;
+        dirname[path_max-1]=0;
       }
       else
         dirname[(p-unshield->filename_pattern)] = 0;
@@ -61,7 +95,11 @@ FILE* unshield_fopen_for_reading(Unshield* unshield, int index, const char* suff
         goto exit;
       }
       else
-        snprintf(filename, sizeof(filename), "%s/%s", dirname, dent->d_name);
+        if(snprintf(filename, path_max, "%s/%s", dirname, dent->d_name)>=path_max)
+        {
+          unshield_error("Pathname exceeds system limits.\n");
+          goto exit;
+        }
     }
     else
       unshield_trace("Could not open directory %s error %s\n", dirname, strerror(errno));
@@ -74,7 +112,8 @@ FILE* unshield_fopen_for_reading(Unshield* unshield, int index, const char* suff
 exit:
     if (sourcedir)
       closedir(sourcedir);
-
+    free(filename);
+    free(dirname);
     return result;
   }
 
