@@ -1,6 +1,7 @@
 /* $Id$ */
 #ifdef __linux__
 #define _BSD_SOURCE 1
+#define _DEFAULT_SOURCE 1
 #define _POSIX_C_SOURCE 2
 #endif
 #include <sys/types.h>
@@ -39,6 +40,10 @@
   #include <limits.h>
 #endif
 
+#ifndef NAME_MAX
+  #define NAME_MAX FILENAME_MAX
+#endif
+
 typedef enum 
 {
   OVERWRITE_ASK,
@@ -117,6 +122,8 @@ static bool make_sure_directory_exists(const char* directory)/*{{{*/
         #endif
         {
           fprintf(stderr, "Failed to create directory %s\n", directory);
+          if(strlen(directory)>NAME_MAX)
+            fprintf(stderr, "Directory name must be less than %i characters\n", NAME_MAX+1);
           goto exit;
         }
       }
@@ -359,8 +366,8 @@ static bool handle_parameters(
 static bool extract_file(Unshield* unshield, const char* prefix, int index)
 {
   bool success;
-  char dirname[256];
-  char filename[256];
+  char* dirname;
+  char* filename;
   char* p;
   int directory = unshield_file_directory(unshield, index);
   long int path_max;
@@ -370,13 +377,15 @@ static bool extract_file(Unshield* unshield, const char* prefix, int index)
   #ifdef PATH_MAX
     path_max = PATH_MAX;
   #else
-    path_max = pathconf(path, _PC_PATH_MAX);
+    path_max = pathconf(prefix, _PC_PATH_MAX);
     if (path_max <= 0)
       path_max = 4096;
   #endif
 
   real_output_directory = malloc(path_max);
   real_filename = malloc(path_max);
+  dirname = malloc(path_max);
+  filename = malloc(path_max);
   if (real_output_directory == NULL || real_filename == NULL)
   {
     fprintf(stderr,"Unable to allocate memory.");
@@ -384,13 +393,35 @@ static bool extract_file(Unshield* unshield, const char* prefix, int index)
     goto exit;
   }
 
-  strcpy(dirname, output_directory);
-  strcat(dirname, "/");
+
+  if(strlen(output_directory) < path_max-1)
+  {
+    strncpy(dirname, output_directory,path_max-1);
+    if (path_max > 0)
+      dirname[path_max - 1]= '\0';
+    strcat(dirname, "/");
+  }
+  else
+  {
+    fprintf(stderr, "\nOutput directory exceeds maximum path length.\n");
+    success = false;
+    goto exit;
+  }
+
 
   if (prefix && prefix[0])
   {
-    strcat(dirname, prefix);
-    strcat(dirname, "/");
+    if(strlen(dirname)+strlen(prefix) < path_max-1)
+    {
+      strcat(dirname, prefix);
+      strcat(dirname, "/");
+    }
+    else
+    {
+    fprintf(stderr, "\nOutput directory exceeds maximum path length.\n");
+    success = false;
+    goto exit;
+    }
   }
 
   if (!junk_paths && directory >= 0)
@@ -398,8 +429,17 @@ static bool extract_file(Unshield* unshield, const char* prefix, int index)
     const char* tmp = unshield_directory_name(unshield, directory);
     if (tmp && tmp[0])
     {
-      strcat(dirname, tmp);
-      strcat(dirname, "/");
+      if(strlen(dirname)+strlen(tmp) < path_max-1)
+      {
+        strcat(dirname, tmp);
+        strcat(dirname, "/");
+      }
+      else
+      {
+      fprintf(stderr, "\nOutput directory exceeds maximum path length.\n");
+      success = false;
+      goto exit;
+      }
     }
   }
 
@@ -447,7 +487,7 @@ static bool extract_file(Unshield* unshield, const char* prefix, int index)
 
   make_sure_directory_exists(dirname);
 
-  snprintf(filename, sizeof(filename), "%s%s", 
+  snprintf(filename, path_max, "%s%s", 
       dirname, unshield_file_name(unshield, index));
 
   for (p = filename + strlen(dirname); *p != '\0'; p++)
@@ -480,11 +520,8 @@ static bool extract_file(Unshield* unshield, const char* prefix, int index)
     fprintf(stderr, "\n\nExtraction failed.\n");
     fprintf(stderr, "Possible directory traversal attack for: %s\n", filename);
     fprintf(stderr, "To be placed at: %s\n\n", real_filename);
-    exit_status = 1;
     success = false;
-    free(real_filename);
-    free(real_output_directory);
-    return success;
+    goto exit;
   }
 
   printf("  extracting: %s\n", filename);
@@ -512,6 +549,8 @@ exit:
   }
   free(real_filename);
   free(real_output_directory);
+  free(dirname);
+  free(filename);
   return success;
 }
 
@@ -629,7 +668,7 @@ static int list_files_helper(Unshield* unshield, const char* prefix, int first,
 
   for (i = first; i <= last; i++)
   {
-    char dirname[256];
+    char dirname[4096];
 
     if (unshield_file_is_valid(unshield, i) && should_process_file(unshield, i))
     {
