@@ -3,7 +3,7 @@ set -e
 cd `dirname $0`
 MD5_FILE=`pwd`/`basename $0 .sh`.md5
 CAB_FILE=`pwd`/data1.cab
-UNSHIELD=/var/tmp/unshield/bin/unshield
+UNSHIELD=${UNSHIELD:-/var/tmp/unshield/bin/unshield}
 
 if [ \! -x ${UNSHIELD} ]; then
     echo "unshield executable not found at $UNSHIELD" >&2
